import pytest
from abstract_open_traffic_generator.port import *


def test_default_port(serializer):
    name = 'Default Port'
    port = Port(name=name)
    print(serializer.json(port))
    assert(port.name == name)
    assert(port.location is None)

def test_physical_port(serializer):
    name = "Physical Port"
    physical = Physical(address='127.0.0.1', board=1, port=1)
    port = Port(name=name, location=Location(physical))
    print(serializer.json(port))
    assert(port.name == name)
    assert(port.location.choice == 'physical')
    assert(port.location.physical.address == '127.0.0.1')


if __name__ == '__main__':
    pytest.main(['-s', __file__])



import pytest


@pytest.fixture(scope='module')
def serializer():
    class Serializer(object):
        def json(self, obj):
            import json
            return json.dumps(obj, indent=2, default=lambda x: x.__dict__)
    return Serializer()            

@pytest.fixture(scope='module')
def b2b_ports():
    from abstract_open_traffic_generator.port import Port
    return [
        Port(name='Tx Port'),
        Port(name='Rx Port')
    ]






class Flow(object):
	"""Flow.Flow class
	
	A high level data plane traffic flow
	Acts as a container for endpoints, frame size, frame rate, duration and packet headers.
	"""
	def __init__(self, name=None, endpoint=None, packet=None, size=None, rate=None):
		self.name = name
		if isinstance(endpoint, (Endpoint, type(None))) is True:
			self.endpoint = endpoint
		else:
			raise TypeError('endpoint must be of type Endpoint')
		self.packet = packet
		if isinstance(size, (Size, type(None))) is True:
			self.size = size
		else:
			raise TypeError('size must be of type Size')
		if isinstance(rate, (Rate, type(None))) is True:
			self.rate = rate
		else:
			raise TypeError('rate must be of type Rate')


class Endpoint(object):
	"""Flow.Endpoint class
	
	An endpoint that dictates the type of flow.
	"""
	_CHOICE_MAP = {
		'PortEndpoint': 'port',
		'DeviceEndpoint': 'device',
	}
	def __init__(self, choice):
		if isinstance(choice, (PortEndpoint, DeviceEndpoint)) is False:
			raise TypeError('choice must be of type: PortEndpoint, DeviceEndpoint')
		self.__setattr__('choice', Endpoint._CHOICE_MAP[choice.__class__.__name__])
		self.__setattr__(Endpoint._CHOICE_MAP[choice.__class__.__name__], choice)


class PortEndpoint(object):
	"""Flow.PortEndpoint class
	
	An endpoint that contains a transmit port and 0..n receive ports.
	"""
	def __init__(self, tx_port=None, rx_ports=None, tx_patterns=None):
		self.tx_port = tx_port
		self.rx_ports = rx_ports
		self.tx_patterns = tx_patterns


class DeviceEndpoint(object):
	"""Flow.DeviceEndpoint class
	
	An endpoint that contains 1..n emulated transmit devices and 1..n  emulated receive devices.
	"""
	def __init__(self, tx_devices=None, rx_devices=None):
		self.tx_devices = tx_devices
		self.rx_devices = rx_devices


class PortPattern(object):
	"""Flow.PortPattern class
	
	A pattern that is applied to a test port
	The name of the pattern will be reflected in the port results.
	"""
	def __init__(self, name=None, offset=None, pattern=None, mask=None):
		self.name = name
		self.offset = offset
		self.pattern = pattern
		self.mask = mask


class Header(object):
	"""Flow.Header class
	
	Container for all traffic packet headers
	"""
	_CHOICE_MAP = {
		'Custom': 'custom',
		'Ethernet': 'ethernet',
		'Vlan': 'vlan',
		'Ipv4': 'ipv4',
		'PfcPause': 'pfcpause',
	}
	def __init__(self, choice):
		if isinstance(choice, (Custom, Ethernet, Vlan, Ipv4, PfcPause)) is False:
			raise TypeError('choice must be of type: Custom, Ethernet, Vlan, Ipv4, PfcPause')
		self.__setattr__('choice', Header._CHOICE_MAP[choice.__class__.__name__])
		self.__setattr__(Header._CHOICE_MAP[choice.__class__.__name__], choice)


class Custom(object):
	"""Flow.Custom class
	
	Custom packet header
	"""
	def __init__(self, bytes=None, patterns=None):
		self.bytes = bytes
		self.patterns = patterns


class BitPattern(object):
	"""Flow.BitPattern class
	
	Container for a bit pattern
	"""
	_CHOICE_MAP = {
		'BitList': 'bitlist',
		'BitCounter': 'bitcounter',
	}
	def __init__(self, choice):
		if isinstance(choice, (BitList, BitCounter)) is False:
			raise TypeError('choice must be of type: BitList, BitCounter')
		self.__setattr__('choice', BitPattern._CHOICE_MAP[choice.__class__.__name__])
		self.__setattr__(BitPattern._CHOICE_MAP[choice.__class__.__name__], choice)


class BitList(object):
	"""Flow.BitList class
	
	A pattern which is a list of values.
	"""
	def __init__(self, offset=None, length=None, count=None, values=None):
		self.offset = offset
		self.length = length
		self.count = count
		self.values = values


class BitCounter(object):
	"""Flow.BitCounter class
	
	An incrementing pattern
	"""
	def __init__(self, offset=None, length=None, count=None, start=None, step=None):
		self.offset = offset
		self.length = length
		self.count = count
		self.start = start
		self.step = step


class Ethernet(object):
	"""Flow.Ethernet class
	
	Ethernet packet header
	"""
	def __init__(self, dst=None, src=None, ether_type=None):
		if isinstance(dst, (StringPattern, type(None))) is True:
			self.dst = dst
		else:
			raise TypeError('dst must be of type StringPattern')
		if isinstance(src, (StringPattern, type(None))) is True:
			self.src = src
		else:
			raise TypeError('src must be of type StringPattern')
		if isinstance(ether_type, (StringPattern, type(None))) is True:
			self.ether_type = ether_type
		else:
			raise TypeError('ether_type must be of type StringPattern')


class StringPattern(object):
	"""Flow.StringPattern class
	
	A string pattern
	"""
	_CHOICE_MAP = {
		'str': 'fixed',
		'list': 'list',
		'StringCounter': 'counter',
	}
	def __init__(self, choice):
		if isinstance(choice, (str, list, StringCounter)) is False:
			raise TypeError('choice must be of type: str, list, StringCounter')
		self.__setattr__('choice', StringPattern._CHOICE_MAP[choice.__class__.__name__])
		self.__setattr__(StringPattern._CHOICE_MAP[choice.__class__.__name__], choice)


class StringCounter(object):
	"""Flow.StringCounter class
	
	TBD
	"""
	def __init__(self, start=None, step=None, direction=None, count=None):
		self.start = start
		self.step = step
		self.direction = direction
		self.count = count


class NumberPattern(object):
	"""Flow.NumberPattern class
	
	A string pattern
	"""
	_CHOICE_MAP = {
		'list': 'list',
		'NumberCounter': 'counter',
	}
	def __init__(self, choice):
		if isinstance(choice, (list, NumberCounter)) is False:
			raise TypeError('choice must be of type: list, NumberCounter')
		self.__setattr__('choice', NumberPattern._CHOICE_MAP[choice.__class__.__name__])
		self.__setattr__(NumberPattern._CHOICE_MAP[choice.__class__.__name__], choice)


class NumberCounter(object):
	"""Flow.NumberCounter class
	
	TBD
	"""
	def __init__(self, start=None, step=None, direction=None, count=None):
		self.start = start
		self.step = step
		self.direction = direction
		self.count = count


class Vlan(object):
	"""Flow.Vlan class
	
	Vlan packet header
	"""
	def __init__(self, priority=None, cfi=None, id=None, protocol=None):
		if isinstance(priority, (StringPattern, type(None))) is True:
			self.priority = priority
		else:
			raise TypeError('priority must be of type StringPattern')
		if isinstance(cfi, (StringPattern, type(None))) is True:
			self.cfi = cfi
		else:
			raise TypeError('cfi must be of type StringPattern')
		if isinstance(id, (StringPattern, type(None))) is True:
			self.id = id
		else:
			raise TypeError('id must be of type StringPattern')
		if isinstance(protocol, (StringPattern, type(None))) is True:
			self.protocol = protocol
		else:
			raise TypeError('protocol must be of type StringPattern')


class Ipv4(object):
	"""Flow.Ipv4 class
	
	Ipv4 packet header
	"""
	def __init__(self, priority=None, src=None, dst=None):
		if isinstance(priority, (Priority, type(None))) is True:
			self.priority = priority
		else:
			raise TypeError('priority must be of type Priority')
		if isinstance(src, (StringPattern, type(None))) is True:
			self.src = src
		else:
			raise TypeError('src must be of type StringPattern')
		if isinstance(dst, (StringPattern, type(None))) is True:
			self.dst = dst
		else:
			raise TypeError('dst must be of type StringPattern')


class PfcPause(object):
	"""Flow.PfcPause class
	
	PFC Pause packet header
	"""
	def __init__(self, dst=None, src=None, ether_type=None, control_op_code=None, priority_enable_vector=None, pfc_queue_0=None, pfc_queue_1=None, pfc_queue_2=None, pfc_queue_3=None, pfc_queue_4=None, pfc_queue_5=None, pfc_queue_6=None, pfc_queue_7=None):
		if isinstance(dst, (StringPattern, type(None))) is True:
			self.dst = dst
		else:
			raise TypeError('dst must be of type StringPattern')
		if isinstance(src, (StringPattern, type(None))) is True:
			self.src = src
		else:
			raise TypeError('src must be of type StringPattern')
		if isinstance(ether_type, (StringPattern, type(None))) is True:
			self.ether_type = ether_type
		else:
			raise TypeError('ether_type must be of type StringPattern')
		if isinstance(control_op_code, (StringPattern, type(None))) is True:
			self.control_op_code = control_op_code
		else:
			raise TypeError('control_op_code must be of type StringPattern')
		if isinstance(priority_enable_vector, (StringPattern, type(None))) is True:
			self.priority_enable_vector = priority_enable_vector
		else:
			raise TypeError('priority_enable_vector must be of type StringPattern')
		if isinstance(pfc_queue_0, (StringPattern, type(None))) is True:
			self.pfc_queue_0 = pfc_queue_0
		else:
			raise TypeError('pfc_queue_0 must be of type StringPattern')
		if isinstance(pfc_queue_1, (StringPattern, type(None))) is True:
			self.pfc_queue_1 = pfc_queue_1
		else:
			raise TypeError('pfc_queue_1 must be of type StringPattern')
		if isinstance(pfc_queue_2, (StringPattern, type(None))) is True:
			self.pfc_queue_2 = pfc_queue_2
		else:
			raise TypeError('pfc_queue_2 must be of type StringPattern')
		if isinstance(pfc_queue_3, (StringPattern, type(None))) is True:
			self.pfc_queue_3 = pfc_queue_3
		else:
			raise TypeError('pfc_queue_3 must be of type StringPattern')
		if isinstance(pfc_queue_4, (StringPattern, type(None))) is True:
			self.pfc_queue_4 = pfc_queue_4
		else:
			raise TypeError('pfc_queue_4 must be of type StringPattern')
		if isinstance(pfc_queue_5, (StringPattern, type(None))) is True:
			self.pfc_queue_5 = pfc_queue_5
		else:
			raise TypeError('pfc_queue_5 must be of type StringPattern')
		if isinstance(pfc_queue_6, (StringPattern, type(None))) is True:
			self.pfc_queue_6 = pfc_queue_6
		else:
			raise TypeError('pfc_queue_6 must be of type StringPattern')
		if isinstance(pfc_queue_7, (StringPattern, type(None))) is True:
			self.pfc_queue_7 = pfc_queue_7
		else:
			raise TypeError('pfc_queue_7 must be of type StringPattern')


class GroupBy(object):
	"""Flow.GroupBy class
	
	Group results 
	"""
	def __init__(self, field=None, label=None):
		self.field = field
		self.label = label


class Size(object):
	"""Flow.Size class
	
	The frame size which overrides the total length of the packet
	"""
	_CHOICE_MAP = {
		'Increment': 'increment',
		'Random': 'random',
	}
	def __init__(self, choice):
		if isinstance(choice, (Increment, Random)) is False:
			raise TypeError('choice must be of type: Increment, Random')
		self.__setattr__('choice', Size._CHOICE_MAP[choice.__class__.__name__])
		self.__setattr__(Size._CHOICE_MAP[choice.__class__.__name__], choice)


class Rate(object):
	"""Flow.Rate class
	
	The rate of packet transmission
	"""
	def __init__(self, type=None, value=None):
		self.type = type
		self.value = value





class Port(object):
	"""Port.Port class
	
	An abstract test port used to associate a unique name with the location of a physical or virtual test location.
	Some different types of test locations are:
	  - physical appliance with multiple ports
	  - physical chassis with multiple cards and ports
	  - local interface
	  - virtual machine
	  - docker container
	"""
	def __init__(self, name=None, location=None):
		self.name = name
		if isinstance(location, (Location, type(None))) is True:
			self.location = location
		else:
			raise TypeError('location must be of type Location')


class Location(object):
	"""Port.Location class
	
	The location of a test resource.
	"""
	_CHOICE_MAP = {
		'Physical': 'physical',
		'Interface': 'interface',
		'Virtual': 'virtual',
		'Container': 'container',
	}
	def __init__(self, choice):
		if isinstance(choice, (Physical, Interface, Virtual, Container)) is False:
			raise TypeError('choice must be of type: Physical, Interface, Virtual, Container')
		self.__setattr__('choice', Location._CHOICE_MAP[choice.__class__.__name__])
		self.__setattr__(Location._CHOICE_MAP[choice.__class__.__name__], choice)


class Physical(object):
	"""Port.Physical class
	
	A physical test port
	"""
	def __init__(self, address=None, board=None, port=None, fanout=None):
		self.address = address
		self.board = board
		self.port = port
		self.fanout = fanout


class Interface(object):
	"""Port.Interface class
	
	An interface test port
	"""
	def __init__(self, name=None):
		self.name = name


class Virtual(object):
	"""Port.Virtual class
	
	A virtual test port
	"""
	def __init__(self, address=None):
		self.address = address


class Container(object):
	"""Port.Container class
	
	A container test port
	"""
	def __init__(self, address=None, port=None):
		self.address = address
		self.port = port





class DeviceGroup(object):
	"""Emulated.DeviceGroup class
	
	An abstract container for emulated device containers.
	"""
	def __init__(self, name=None, ports=None, devices=None):
		self.name = name
		self.ports = ports
		self.devices = devices


class Device(object):
	"""Emulated.Device class
	
	An abstract container for emulated protocols.
	"""
	def __init__(self, name=None, devices_per_port=None, parent=None, protocols=None):
		self.name = name
		self.devices_per_port = devices_per_port
		self.parent = parent
		self.protocols = protocols


class Ethernet(object):
	"""Emulated.Ethernet class
	
	Emulated ethernet protocol
	"""
	def __init__(self, name=None, mac=None, mtu=None):
		self.name = name
		self.mac = mac
		self.mtu = mtu


class Vlan(object):
	"""Emulated.Vlan class
	
	Emulated vlan protocol
	"""
	def __init__(self, name=None, parent=None, tpid=None, priority=None, id=None):
		self.name = name
		self.parent = parent
		self.tpid = tpid
		self.priority = priority
		self.id = id


class Ipv4(object):
	"""Emulated.Ipv4 class
	
	Emulated ipv4 protocol
	"""
	def __init__(self, name=None, address=None, gateway=None, prefix=None):
		self.name = name
		self.address = address
		self.gateway = gateway
		self.prefix = prefix


class Bgpv4(object):
	"""Emulated.Bgpv4 class
	
	Emulated bgpv4 protocol
	"""
	def __init__(self, name=None, as_number_2_byte=None, dut_as_number_2_byte=None, as_number_4_byte=None, as_number_set_mode=None, type=None, hold_time_interval=None, keep_alive_interval=None, graceful_restart=None, authentication=None, ttl=None, dut_ipv4_address=None):
		self.name = name
		self.as_number_2_byte = as_number_2_byte
		self.dut_as_number_2_byte = dut_as_number_2_byte
		self.as_number_4_byte = as_number_4_byte
		self.as_number_set_mode = as_number_set_mode
		self.type = type
		self.hold_time_interval = hold_time_interval
		self.keep_alive_interval = keep_alive_interval
		self.graceful_restart = graceful_restart
		self.authentication = authentication
		self.ttl = ttl
		self.dut_ipv4_address = dut_ipv4_address





class Priority(object):
	"""Flow.Ipv4.Priority class
	
	Ipv4 ip priority that can be one of RAW or DSCP.
	"""
	_CHOICE_MAP = {
		'Dscp': 'dscp',
		'NumberPattern': 'raw',
	}
	def __init__(self, choice):
		if isinstance(choice, (Dscp, NumberPattern)) is False:
			raise TypeError('choice must be of type: Dscp, NumberPattern')
		self.__setattr__('choice', Priority._CHOICE_MAP[choice.__class__.__name__])
		self.__setattr__(Priority._CHOICE_MAP[choice.__class__.__name__], choice)


class Dscp(object):
	"""Flow.Ipv4.Dscp class
	
	Differentiated services code point (DSCP) packet field.
	Per hop bevaior (PHB) property constraint is 6 bits
	>=0 <=63
	The following are valid PHB values:
	  - PHB_DEFAULT = 0
	  - PHB_CS1 = 8
	  - PHB_CS2 = 16
	  - PHB_CS3 = 24
	  - PHB_CS4 = 32
	  - PHB_CS5 = 40
	  - PHB_CS6 = 48
	  - PHB_CS7 = 56
	  - PHB_EF46 = 46
	  - PHB_AF11 = 10
	  - PHB_AF12 = 12
	  - PHB_AF13 = 14
	  - PHB_AF21 = 18
	  - PHB_AF22 = 20
	  - PHB_AF23 = 22
	  - PHB_AF31 = 26
	  - PHB_AF32 = 28
	  - PHB_AF33 = 30
	  - PHB_AF41 = 24
	  - PHB_AF42 = 36
	  - PHB_AF43 = 38
	
	Explicit congestion notification (ECN) packet field
	ECN value constraint is 2 bits
	>=0 <=3 The following are valid explicit-congestion-notification (ECN) values:
	  - ECN_NON_CAPABLE = 0
	  - ECN_CAPABLE = 1
	  - ECN_CAPABLE = 2
	  - ECN_CONGESTION_ENCOUNTERED = 3
	"""
	def __init__(self, phb=None, ecn=None):
		if isinstance(phb, (NumberPattern, type(None))) is True:
			self.phb = phb
		else:
			raise TypeError('phb must be of type NumberPattern')
		if isinstance(ecn, (NumberPattern, type(None))) is True:
			self.ecn = ecn
		else:
			raise TypeError('ecn must be of type NumberPattern')





class Config(object):
	"""State.Config class
	
	placeholder
	"""
	def __init__(self, state=None, ports=None, devices=None, flows=None):
		self.state = state
		self.ports = ports
		self.devices = devices
		self.flows = flows


class Flow(object):
	"""State.Flow class
	
	Request for the traffic generator to move flows to a specific state.
	"""
	def __init__(self, state=None, flows=None):
		self.state = state
		self.flows = flows


class Capture(object):
	"""State.Capture class
	
	placeholder
	"""
	def __init__(self, requested_state=None, captures=None):
		self.requested_state = requested_state
		self.captures = captures


class Desired(object):
	"""State.Desired class
	
	The desired state of the traffic generator 
	"""
	def __init__(self, configuration=None, traffic=None, capture=None):
		if isinstance(configuration, (Config, type(None))) is True:
			self.configuration = configuration
		else:
			raise TypeError('configuration must be of type Config')
		if isinstance(traffic, (Flow, type(None))) is True:
			self.traffic = traffic
		else:
			raise TypeError('traffic must be of type Flow')
		if isinstance(capture, (Capture, type(None))) is True:
			self.capture = capture
		else:
			raise TypeError('capture must be of type Capture')


class Actual(object):
	"""State.Actual class
	
	The desired and actual state of the traffic generator
	"""
	def __init__(self, states=None, flow=None, capture=None):
		self.states = states
		self.flow = flow
		self.capture = capture





class Increment(object):
	"""Flow.Size.Increment class
	
	Frame size that increments from a starting size to  an ending size incrementing by a step size
	"""
	def __init__(self, start=None, end=None, step=None):
		self.start = start
		self.end = end
		self.step = step


class Random(object):
	"""Flow.Size.Random class
	
	Random frame size
	"""
	def __init__(self, min=None, max=None):
		self.min = min
		self.max = max
